import React from 'react';
import PropTypes from 'prop-types';
import $ from 'lodash';
import _regeneratorRuntime from 'babel-runtime/regenerator';
import KOS$1 from 'kos-core';

/**
 * 表单校验数据常量
 */
var XFORM_VALIDATE_DATA = '@@xform-validate-data';

/**
 * 表单控件可见数据常量
 */
var XFORM_DISPLAY_DATA = '@@xform-display-data';

/**
 * 表单变更
 */
var XFORM_FIELD_CHANGE = '@@xform-field-change';

var XFORM_VALIDATE = '@@xfrom-validate';

var asyncToGenerator = function (fn) {
  return function () {
    var gen = fn.apply(this, arguments);
    return new Promise(function (resolve, reject) {
      function step(key, arg) {
        try {
          var info = gen[key](arg);
          var value = info.value;
        } catch (error) {
          reject(error);
          return;
        }

        if (info.done) {
          resolve(value);
        } else {
          return Promise.resolve(value).then(function (value) {
            step("next", value);
          }, function (err) {
            step("throw", err);
          });
        }
      }

      return step("next");
    });
  };
};

var classCallCheck = function (instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
};

var createClass = function () {
  function defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }

  return function (Constructor, protoProps, staticProps) {
    if (protoProps) defineProperties(Constructor.prototype, protoProps);
    if (staticProps) defineProperties(Constructor, staticProps);
    return Constructor;
  };
}();

var defineProperty = function (obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
};

var _extends = Object.assign || function (target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i];

    for (var key in source) {
      if (Object.prototype.hasOwnProperty.call(source, key)) {
        target[key] = source[key];
      }
    }
  }

  return target;
};

var inherits = function (subClass, superClass) {
  if (typeof superClass !== "function" && superClass !== null) {
    throw new TypeError("Super expression must either be null or a function, not " + typeof superClass);
  }

  subClass.prototype = Object.create(superClass && superClass.prototype, {
    constructor: {
      value: subClass,
      enumerable: false,
      writable: true,
      configurable: true
    }
  });
  if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass;
};

var possibleConstructorReturn = function (self, call) {
  if (!self) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  }

  return call && (typeof call === "object" || typeof call === "function") ? call : self;
};

var slicedToArray = function () {
  function sliceIterator(arr, i) {
    var _arr = [];
    var _n = true;
    var _d = false;
    var _e = undefined;

    try {
      for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
        _arr.push(_s.value);

        if (i && _arr.length === i) break;
      }
    } catch (err) {
      _d = true;
      _e = err;
    } finally {
      try {
        if (!_n && _i["return"]) _i["return"]();
      } finally {
        if (_d) throw _e;
      }
    }

    return _arr;
  }

  return function (arr, i) {
    if (Array.isArray(arr)) {
      return arr;
    } else if (Symbol.iterator in Object(arr)) {
      return sliceIterator(arr, i);
    } else {
      throw new TypeError("Invalid attempt to destructure non-iterable instance");
    }
  };
}();

/**
 * 创建校验信息更新的payloads
 * @param {Obeject} payload 触发的action
 * @param {Object} state State
 * @param {Object} resultMap {
 *  name:{
 *    help:'校验失败',
 *    status:'error',
 *    hasFeedback:true
 *  }
 * }结果
 */
var createValidatePayload = function createValidatePayload(payload, getState, resultMap) {
  var formName = payload.formName;

  var newPayload = {};
  var state = getState();

  var stateValidateData = _extends({}, state[XFORM_VALIDATE_DATA]);
  var formValidateData = _extends({}, stateValidateData[formName], resultMap);

  stateValidateData[formName] = formValidateData;
  newPayload[XFORM_VALIDATE_DATA] = stateValidateData;
  return newPayload;
};

/**
 * 根据表单名称和字段名称获取校验信息
 * @param {Object} state State,store.getState()[namespace]获取的值
 * @param {String} formName 表单名称
 * @param {String} field 字段名称
 */
var getFieldValidateData = function getFieldValidateData(state, formName, field) {
  var formValidateData = getFormValidateData(state, formName);
  return formValidateData[field];
};

/**
 * 根据表单名称获取校验信息
 * @param {Object} state State,store.getState()[namespace]获取的值
 * @param {String} formName 表单名称
 * @param {String} field 字段名称
 */
var getFormValidateData = function getFormValidateData(state, formName) {
  var formValidateData = state[XFORM_VALIDATE_DATA] || {};
  return formValidateData[formName] || {};
};

/**
 * 创建值更新的payload
 * @param {Object} payload 触发的action
 * @param {Object} state State
 */
var createFieldValuePayload = function createFieldValuePayload(payload, getState) {
  var field = payload.field,
      value = payload.value,
      formName = payload.formName;

  var newPayload = {};
  var state = getState();

  if (formName) {
    newPayload[formName] = _extends({}, state[formName]);

    newPayload[formName][field] = value;
  } else {
    newPayload[field] = value;
  }
  return newPayload;
};

/**
 * 创建校验信息更新的payloads
 * @param {Obeject} payload 触发的action
 * @param {Object} state State
 * @param {Object} result {a:true,b:false}结果
 */
var createDisplayPayload = function createDisplayPayload(payload, getState, result) {
  var formName = payload.formName;

  var newPayload = {};
  var state = getState();

  var stateDisplayData = _extends({}, state[XFORM_DISPLAY_DATA]);
  var formDisplayData = _extends({}, stateDisplayData[formName], result);

  stateDisplayData[formName] = formDisplayData;
  newPayload[XFORM_DISPLAY_DATA] = stateDisplayData;
  return newPayload;
};

/**
 * 根据表单名称和字段名称获取展示信息
 * @param {Object} state State,store.getState()[namespace]获取的值
 * @param {String} formName 表单名称
 * @param {String} field 字段名称
 */
var getFieldDisplayData = function getFieldDisplayData(state, formName, field) {
  var formValidateData = state[XFORM_DISPLAY_DATA] || {};
  formValidateData = formValidateData[formName] || {};

  return formValidateData[field];
};

var _arguments = arguments;

var formInsMap = {};
var addForm = function addForm(ins) {
  var namespace = ins.context.getNamespace();
  var formName = ins.props.name;

  formInsMap[namespace + "/" + formName] = ins;
};
var removeForm = function removeForm(namespace, formName) {
  delete formInsMap[namespace + "/" + formName];
};
var getForm = function getForm(namespace, formName) {
  return formInsMap[namespace + "/" + formName];
};
var runFormMethod = function runFormMethod(namespace, formName, method) {
  var ins = getForm(namespace, formName);

  if (ins) {
    return ins[method] && ins[method]();
  }
};
var validate = function validate(namespace, formName, callback) {
  if (typeof formName == "function" && _arguments.length == 2) {
    callback = formName;
    formName = undefined;
  }
  var ins = getForm(namespace, formName);

  if (ins) {
    ins.validate(callback);
  }
};
var getFormData = function getFormData(namespace, formName) {
  return runFormMethod(namespace, formName, "getData");
};
var resetForm = function resetForm(namespace, formName) {
  return runFormMethod(namespace, formName, "reset");
};

var Form = function (_React$Component) {
  inherits(Form, _React$Component);

  function Form() {
    classCallCheck(this, Form);
    return possibleConstructorReturn(this, (Form.__proto__ || Object.getPrototypeOf(Form)).apply(this, arguments));
  }

  createClass(Form, [{
    key: "dispatch",
    value: function dispatch(action) {
      var dispatch = this.props.dispatch || this.context.dispatch;

      dispatch && dispatch(action);
    }
  }, {
    key: "getState",
    value: function getState() {
      var getState = this.context.getState;

      if (getState) {
        return getState();
      }

      return this.props;
    }
    /**
     * 根据字段名称，获取字段的展示数据
     * @param {String} field 字段名称
     */

  }, {
    key: "getFieldDisplayData",
    value: function getFieldDisplayData$$1(field) {
      var formName = this.props.name;

      var state = this.getState();
      return getFieldDisplayData(state, formName, field);
    }
    /**
     * 根据字段名称，获取字段的校验数据
     * @param {String} field 字段名称
     */

  }, {
    key: "getFieldVaidateData",
    value: function getFieldVaidateData(field) {
      var formName = this.props.name;

      var state = this.getState();
      return getFieldValidateData(state, formName, field);
    }
    /**
     * 字段值发生变化时触发
     * @param {String} field 字段名称
     * @param {String} value 值
     */

  }, {
    key: "onFieldChange",
    value: function onFieldChange(field, value) {
      var formName = this.props.name;


      this.dispatch({
        type: XFORM_FIELD_CHANGE,
        payload: {
          field: field,
          value: value,
          formName: formName
        }
      });
    }
    /**
     * 获取表单值
     */

  }, {
    key: "getData",
    value: function getData() {
      var formName = this.props.name;

      var state = this.getState();

      return state[formName] || {};
    }
  }, {
    key: "reset",
    value: function reset() {
      var namespace = this.context.getNamespace();
      var formName = this.props.name;

      var Model = KOS.getModel(namespace);

      var initialModel = Model.getInitial();
      this.setData(initialModel[formName]);
    }
  }, {
    key: "setData",
    value: function setData(formData) {
      var formName = this.props.name;


      this.dispatch({
        type: "setState",
        payload: defineProperty({}, formName, formData)
      });
    }
    /**
     * 根据字段名称，获取字段的值
     * @param {String} field 字段名称
     */

  }, {
    key: "getFieldValue",
    value: function getFieldValue(field) {
      var formData = this.getData();
      return formData[field];
    }
  }, {
    key: "validate",
    value: function validate(callback) {
      var formName = this.props.name;

      this.dispatch({
        type: XFORM_VALIDATE,
        payload: {
          formName: formName,
          callback: callback
        }
      });
    }
  }, {
    key: "onSubmit",
    value: function onSubmit() {
      var _this2 = this;

      var onSubmit = this.props.onSubmit;


      this.validate(function (result) {
        var formData = _this2.getData();
        result && onSubmit && onSubmit(formData);
      });

      return false;
    }
  }, {
    key: "getChildContext",
    value: function getChildContext() {
      return {
        onFieldChange: this.onFieldChange.bind(this),
        getFieldValue: this.getFieldValue.bind(this),
        getFieldVaidateData: this.getFieldVaidateData.bind(this),
        getFieldDisplayData: this.getFieldDisplayData.bind(this)
      };
    }
  }, {
    key: "componentDidMount",
    value: function componentDidMount() {
      addForm(this);
    }
  }, {
    key: "componentWillUnmount",
    value: function componentWillUnmount() {
      var namespace = this.context.getNamespace();
      var formName = this.props.name;


      removeForm(namespace, formName);
    }
  }, {
    key: "render",
    value: function render() {
      var _this3 = this;

      var children = this.props.children;

      return React.createElement(
        "form",
        {
          onSubmit: function onSubmit() {
            return _this3.onSubmit();
          }
        },
        children
      );
    }
  }]);
  return Form;
}(React.Component);

// 传递给field用的


Form.propTypes = {
  name: PropTypes.string.isRequired,
  onSubmit: PropTypes.func,
  dispatch: PropTypes.func
};
Form.childContextTypes = {
  onFieldChange: PropTypes.func,
  getFieldValue: PropTypes.func,
  getFieldVaidateData: PropTypes.func,
  getFieldDisplayData: PropTypes.func
};

// 来自wrapper
Form.contextTypes = {
  dispatch: PropTypes.func,
  getState: PropTypes.func,
  getNamespace: PropTypes.func
};

Form.getForm = getForm;
Form.getFormData = getFormData;
Form.resetForm = resetForm;
Form.validate = validate;

var xField = (function (_ref) {
  var FieldWrapper = _ref.FieldWrapper,
      FieldProps = _ref.FieldProps;

  var Field = function (_React$PureComponent) {
    inherits(Field, _React$PureComponent);

    function Field(props) {
      classCallCheck(this, Field);

      var _this2 = possibleConstructorReturn(this, (Field.__proto__ || Object.getPrototypeOf(Field)).call(this, props));

      _this2.state = {};

      _this2.onFieldChange = _this2.fieldChange.bind(_this2);
      _this2.getFieldValue = _this2.getValue.bind(_this2);
      return _this2;
    }

    createClass(Field, [{
      key: 'getValue',
      value: function getValue() {
        var field = this.props.field;


        return this.isOnComposition ? this.state.inputValue : this.context.getFieldValue(field);
      }
    }, {
      key: 'getValidateData',
      value: function getValidateData() {
        var field = this.props.field;

        return this.context.getFieldVaidateData(field);
      }
    }, {
      key: 'getDisplay',
      value: function getDisplay() {
        var field = this.props.field;

        return this.context.getFieldDisplayData(field) === false ? false : true;
      }
    }, {
      key: 'fieldChange',
      value: function fieldChange(onChange) {
        return function (e) {
          var _props = this.props,
              field = _props.field,
              getOnChangeValue = _props.getOnChangeValue;

          var value = getOnChangeValue.apply(this, arguments);

          if (this.isOnComposition) {
            this.innerValue = value;
            this.setState({
              inputValue: value
            });
          } else {
            onChange && onChange.apply(this, arguments);
            this.context.onFieldChange(field, value);
          }
        }.bind(this);
      }
    }, {
      key: 'onCompositionHandler',
      value: function onCompositionHandler(type, e) {
        switch (type) {
          case 'start':
            this.isOnComposition = true;
            break;
          case 'update':
            this.isOnComposition = true;
            break;
          case 'end':
            this.isOnComposition = false;
            if (e.target instanceof HTMLInputElement && !this.isOnComposition) {
              this.onChange(e);
            }
            break;
        }
      }
    }, {
      key: 'render',
      value: function render() {
        var _this3 = this;

        var isDisplay = this.getDisplay();
        if (!isDisplay) {
          return null;
        }

        var children = this.props.children;

        var validateData = this.getValidateData();

        var fieldProps = _extends({}, FieldProps, this.props, validateData);

        return React.createElement(
          FieldWrapper,
          fieldProps,
          React.Children.map(children, function (child) {
            var onChange = child.props.onChange;

            _this3.onChange = _this3.onFieldChange(onChange);
            var props = _extends({}, child.props, {
              onChange: _this3.onChange,
              onCompositionStart: _this3.onCompositionHandler.bind(_this3, 'start'),
              onCompositionUpdate: _this3.onCompositionHandler.bind(_this3, 'update'),
              onCompositionEnd: _this3.onCompositionHandler.bind(_this3, 'end'),
              value: _this3.getFieldValue()
            });
            return React.createElement(child.type, props);
          })
        );
      }
    }]);
    return Field;
  }(React.PureComponent);

  Field.defaultProps = {
    getOnChangeValue: function getOnChangeValue(e) {
      return e.target.value;
    }
  };


  Field.contextTypes = {
    onFieldChange: PropTypes.func,
    getFieldValue: PropTypes.func,
    getFieldVaidateData: PropTypes.func,
    getFieldDisplayData: PropTypes.func
  };

  return Field;
});

var formatStr = function formatStr(str, data) {
  data = [].concat(data);
  return str.replace(/\{(\d+)\}/g, function (m, i) {
    return data[i];
  });
};
var getValidateHelp = function getValidateHelp(help, status, data) {
  var h = '';
  if ($.isString(help)) {
    h = status === 'success' ? '' : help;
  } else {
    h = help && help[status];
  }

  return formatStr(h, data);
};

var _this = undefined;

/**
 * 支持的校验配置规则：
[{
  field:'name',
  rules:['required']
},{
  field:'name',
  rules:['maxLength:10']
},{
  field:'email',
  rules:['email']
},{
  field:'age',
  rules:[/d+/],
  help:'必须是数字'
},{
  field:'regpassword',
  data:[1234],
  fn(state,{field,value,formName},data){
    return true;
  }
}]
 */

var RuleParseUtil = {
  /**
   * 解析stringRule
   * @param {String} rule 规则，如：maxLength:2@最大长度不能超过{0};minLength min:1;rangeLength:1,2
   */
  stringParser: function stringParser(rule) {
    if (!rule) {
      return [];
    }
    rule = rule.split(';');
    var list = [];

    rule.forEach(function (item) {
      var _item$split = item.split('@'),
          _item$split2 = slicedToArray(_item$split, 2),
          express = _item$split2[0],
          help = _item$split2[1];

      var _express$split = express.split(':'),
          _express$split2 = slicedToArray(_express$split, 2),
          name = _express$split2[0],
          data = _express$split2[1];

      var v = _extends({}, getRule(name));

      // 转换成数组
      if (data) {
        v.data = data.split(',');
      }
      if (help) {
        v.help = help;
      }

      list.push(v);
    });

    return list;
  },
  functionParser: function functionParser(validator) {
    return {
      name: 'fn',
      fn: validator
    };
  },
  regexpParser: function regexpParser(validator) {
    return _extends({}, getRule('regexp'), {
      data: validator
    });
  },
  objectParser: function objectParser(validator) {
    var name = validator.name,
        help = validator.help,
        fn = validator.fn;


    if (name) {
      return _extends({}, getRule(name), validator);
    }

    return _extends({
      name: 'custome'
    }, validator);
  }
};

var parseRule = function parseRule(rule, custHelp) {
  var v = null;
  if ($.isString(rule)) {
    v = RuleParseUtil.stringParser(rule);
  } else if ($.isFunction(rule)) {
    v = RuleParseUtil.functionParser(rule);
  } else if ($.isRegExp(rule)) {
    v = RuleParseUtil.regexpParser(rule);
  } else if ($.isObject(rule)) {
    v = RuleParseUtil.objectParser(rule);
  }

  if (v && custHelp !== undefined) {
    v.help = custHelp;
  }

  return v;
};

var parseValidatorRules = function parseValidatorRules(validatorItem) {
  var field = validatorItem.field,
      help = validatorItem.help;

  var list = [];
  var rules = [].concat(validatorItem.rules);

  rules.forEach(function (ruleItem) {
    list = list.concat(parseRule(ruleItem, help));
  });

  return list;
};

var validatorsMap = {};
var addRule = function addRule(name, fn, help) {
  validatorsMap[name] = {
    name: name, fn: fn, help: help
  };
};

var removeRule = function removeRule(name) {
  delete validatorsMap[name];
};

var getRule = function getRule(name) {
  return validatorsMap[name];
};

/**
 *
 * @param {Object} validator 校验器
 * @param {Object} payload 参数
 * @param {Object} state 当前namespace下的Model
 */
var runRule = function () {
  var _ref = asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee(validator, payload, getState) {
    var fn, help, data, field, value, formName, result, validateStatus, runnerHelp, fieldResult;
    return _regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            fn = validator.fn, help = validator.help, data = validator.data;
            field = payload.field, value = payload.value, formName = payload.formName;
            _context.next = 4;
            return fn(getState, { value: value, field: field, formName: formName }, data);

          case 4:
            result = _context.sent;
            validateStatus = 'success';
            runnerHelp = '';

            if (!$.isBoolean(result)) {
              _context.next = 11;
              break;
            }

            validateStatus = result ? 'success' : 'error';
            _context.next = 18;
            break;

          case 11:
            if (!$.isString(result)) {
              _context.next = 16;
              break;
            }

            // 返回的是错误消息
            runnerHelp = result;
            if (result) {
              validateStatus = result ? 'error' : 'success';
            }
            _context.next = 18;
            break;

          case 16:
            if (!$.isObject(result)) {
              _context.next = 18;
              break;
            }

            return _context.abrupt('return', result);

          case 18:
            fieldResult = {
              validateStatus: validateStatus,
              hasFeedback: true,
              help: runnerHelp || getValidateHelp(help, validateStatus, data)
            };
            return _context.abrupt('return', fieldResult);

          case 20:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, _this);
  }));

  return function runRule(_x, _x2, _x3) {
    return _ref.apply(this, arguments);
  };
}();

var ValidatorRule = /*#__PURE__*/Object.freeze({
  parseRule: parseRule,
  parseValidatorRules: parseValidatorRules,
  addRule: addRule,
  removeRule: removeRule,
  getRule: getRule,
  runRule: runRule
});

var FieldValidator = function () {
  function FieldValidator(field, validators) {
    classCallCheck(this, FieldValidator);

    this.field = field;
    this.validators = validators;
    this.disabled = false;

    this.list = this.initValidator(validators);
  }

  createClass(FieldValidator, [{
    key: 'initValidator',
    value: function initValidator(validators) {
      var list = [];
      validators.forEach(function (item) {
        list = list.concat(item);
      });
      return list;
    }
  }, {
    key: 'setDisable',
    value: function setDisable() {
      this.disabled = true;
    }
  }, {
    key: 'setEnable',
    value: function setEnable() {
      this.disabled = false;
    }
  }, {
    key: 'isDisable',
    value: function isDisable(state, formName, field) {
      return getFieldDisplayData(state, formName, field);
    }
  }, {
    key: 'run',
    value: function () {
      var _ref = asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee(payload, getState) {
        var formName, field, state, disable, validateResult, list, i, len, validator, result;
        return _regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                // TODO：如果表单为隐藏的，则不执行校验器，这部分逻辑有耦合
                formName = payload.formName, field = payload.field;
                state = getState();
                disable = this.isDisable(state, formName, field);

                if (!(disable === false)) {
                  _context.next = 5;
                  break;
                }

                return _context.abrupt('return', validateResult);

              case 5:
                validateResult = null;
                list = this.list;
                i = 0, len = list.length;

              case 8:
                if (!(i < len)) {
                  _context.next = 20;
                  break;
                }

                validator = list[i];

                if (!(validator && validator.fn)) {
                  _context.next = 17;
                  break;
                }

                _context.next = 13;
                return runRule(validator, payload, getState);

              case 13:
                result = _context.sent;

                if (!(result.validateStatus === 'error')) {
                  _context.next = 17;
                  break;
                }

                validateResult = result;
                return _context.abrupt('break', 20);

              case 17:
                i++;
                _context.next = 8;
                break;

              case 20:
                return _context.abrupt('return', validateResult);

              case 21:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      function run(_x, _x2) {
        return _ref.apply(this, arguments);
      }

      return run;
    }()
  }]);
  return FieldValidator;
}();

FieldValidator.factory = function (validators) {
  var vas = {};

  validators.forEach(function (validatorItem) {
    var field = validatorItem.field;

    var list = parseValidatorRules(validatorItem);

    vas[field] = vas[field] || [];
    vas[field] = vas[field].concat(list);
  });

  // 转换成对象进行管理
  for (var field in vas) {
    if (field) {
      vas[field] = new FieldValidator(field, vas[field]);
    }
  }

  return vas;
};

var Validator = function () {
  function Validator() {
    var validators = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];
    var namespace = arguments[1];
    var formName = arguments[2];
    classCallCheck(this, Validator);

    this.formName = formName;
    this.namespace = namespace;

    this.fieldValidateIns = FieldValidator.factory(validators);
  }

  createClass(Validator, [{
    key: 'getValidatorsByField',
    value: function getValidatorsByField(field) {
      return this.fieldValidateIns[field] || null;
    }
  }, {
    key: 'runAll',
    value: function () {
      var _ref = asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee(getState) {
        var formName, fieldValidateIns, state, formData, fieldResult, formResult, field, value, result;
        return _regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                formName = this.formName, fieldValidateIns = this.fieldValidateIns;
                state = getState();
                formData = formName ? state[formName] : state;

                formData = formData || {};

                fieldResult = {};
                formResult = true;
                _context.t0 = _regeneratorRuntime.keys(fieldValidateIns);

              case 7:
                if ((_context.t1 = _context.t0()).done) {
                  _context.next = 17;
                  break;
                }

                field = _context.t1.value;
                value = formData[field];
                _context.next = 12;
                return this.run({
                  field: field,
                  value: value,
                  formName: formName
                }, getState);

              case 12:
                result = _context.sent;


                formResult = formResult && (result ? result.validateStatus !== 'error' : true);
                fieldResult[field] = result;
                _context.next = 7;
                break;

              case 17:
                return _context.abrupt('return', {
                  formResult: formResult,
                  fieldResult: fieldResult
                });

              case 18:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      function runAll(_x2) {
        return _ref.apply(this, arguments);
      }

      return runAll;
    }()
  }, {
    key: 'run',
    value: function () {
      var _ref2 = asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee2(payload, getState) {
        var field, fieldValidate;
        return _regeneratorRuntime.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                field = payload.field;
                fieldValidate = this.getValidatorsByField(field);

                if (fieldValidate) {
                  _context2.next = 4;
                  break;
                }

                return _context2.abrupt('return', null);

              case 4:
                return _context2.abrupt('return', fieldValidate.run(payload, getState));

              case 5:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));

      function run(_x3, _x4) {
        return _ref2.apply(this, arguments);
      }

      return run;
    }()
  }]);
  return Validator;
}();

Validator.create = function (validatorConfig, namespace) {
  var map = {};
  var validatorMap = {};
  validatorConfig.forEach(function (item) {
    var formName = item.formName,
        validators = item.validators;


    var fvList = map[formName] || [];
    if (validators) {
      fvList = fvList.concat(validators);
    } else {
      fvList = fvList.concat(item);
    }
    map[formName] = fvList;
  });

  for (var formName in map) {
    // 初始化
    validatorMap[formName] = new Validator(map[formName], namespace, formName);
  }

  return validatorMap;
};

var _this$1 = undefined;

var KOSUtil = KOS$1.Util;

/**
 * 获取Model的validatorIns
 * @param {Object} model model
 * @param {String} formName 表单名
 */
var getModelValidatorIns = function getModelValidatorIns(model, formName) {
  var validatorMap = model.validatorMap;

  var namespace = model.getNamespace();

  // 此处有性能优化的空间，如果一个Model下存在多个formName的时候
  if (!validatorMap) {
    var validatorConfig = model.getAttr('validators');
    model.validatorMap = validatorMap = validatorConfig ? Validator.create(validatorConfig, namespace) : {};
  }

  return validatorMap[formName];
};

/**
 * 
 * @param {Function} dispatch dispatch方法，包装了namesapce的
 * @param {Function} getState 获取当前namespace下的getState方法
 * @param {Object} action 包含type、payload的对象实体
 */
var fieldValidateMiddleware = function () {
  var _ref = asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee(dispatch, getState, action) {
    var _KOSUtil$getActionTyp, namespace, type, model, payload, formName, field, validatorIns, result, fieldResult;

    return _regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _KOSUtil$getActionTyp = KOSUtil.getActionType(action.type), namespace = _KOSUtil$getActionTyp.namespace, type = _KOSUtil$getActionTyp.type;
            model = KOS$1.getModel(namespace);
            payload = action.payload;
            formName = payload.formName, field = payload.field;

            // 有表单校验相关配置

            validatorIns = getModelValidatorIns(model, formName);

            if (!validatorIns) {
              _context.next = 13;
              break;
            }

            _context.next = 8;
            return validatorIns.run(payload, getState);

          case 8:
            result = _context.sent;
            fieldResult = {};

            fieldResult[field] = result;

            // 触发表单校验结果更新
            dispatch({
              type: 'setState',
              payload: createValidatePayload(payload, getState, fieldResult)
            });

            return _context.abrupt('return', fieldResult);

          case 13:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, _this$1);
  }));

  return function fieldValidateMiddleware(_x, _x2, _x3) {
    return _ref.apply(this, arguments);
  };
}();

/**
 * 
 * @param {Function} dispatch dispatch方法，包装了namesapce的
 * @param {Function} getState 获取当前namespace下的getState方法
 * @param {Object} action 包含type、payload的对象实体
 */
var formValidateMiddleware = function () {
  var _ref2 = asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee2(dispatch, getState, action) {
    var _KOSUtil$getActionTyp2, namespace, type, model, payload, formName, field, validatorIns, formResult, result, callback;

    return _regeneratorRuntime.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            _KOSUtil$getActionTyp2 = KOSUtil.getActionType(action.type), namespace = _KOSUtil$getActionTyp2.namespace, type = _KOSUtil$getActionTyp2.type;
            model = KOS$1.getModel(namespace);
            payload = action.payload;
            formName = payload.formName, field = payload.field;
            validatorIns = getModelValidatorIns(model, formName);
            formResult = true;

            if (!validatorIns) {
              _context2.next = 12;
              break;
            }

            _context2.next = 9;
            return validatorIns.runAll(getState);

          case 9:
            result = _context2.sent;

            formResult = result.formResult;

            // 触发表单校验结果更新
            dispatch({
              type: 'setState',
              payload: createValidatePayload(payload, getState, result.fieldResult)
            });

          case 12:

            // 校验后进行回调
            callback = payload.callback;

            callback && callback(formResult);

          case 14:
          case 'end':
            return _context2.stop();
        }
      }
    }, _callee2, _this$1);
  }));

  return function formValidateMiddleware(_x4, _x5, _x6) {
    return _ref2.apply(this, arguments);
  };
}();

var addRule$1 = addRule;


addRule$1('required', function (getState, _ref, required) {
  var value = _ref.value,
      field = _ref.field;

  return !!value;
}, '不能为空！');

addRule$1('maxLength', function (getState, _ref2, maxLength) {
  var value = _ref2.value,
      field = _ref2.field;

  if (value) {
    return value.length <= maxLength;
  }
  return true;
});

addRule$1('minLength', function (getState, _ref3, minLength) {
  var value = _ref3.value,
      field = _ref3.field;

  if (value) {
    return value.length >= minLength;
  }
  return true;
});

addRule$1('regexp', function (getState, _ref4, regExp) {
  var value = _ref4.value,
      field = _ref4.field;

  return regExp.test(value);
});

var FormValidator = _extends({
  /**
   * 校验器类
   */
  Validator: Validator,
  /**
   * 表单字段校验器中间件
   */
  fieldValidateMiddleware: fieldValidateMiddleware,
  /**
   * 表单校验器中间件
   */
  formValidateMiddleware: formValidateMiddleware

}, ValidatorRule);

var _this$2 = undefined;

var KOSUtil$1 = KOS$1.Util;
/**
 * 
 * @param {Function} dispatch dispatch方法，包装了namesapce的
 * @param {Function} getState 获取当前namespace下的getState方法
 * @param {Object} action 包含type、payload的对象实体
 */
var fieldDisplayMiddleware = function () {
  var _ref = asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee(dispatch, getState, action) {
    var _KOSUtil$getActionTyp, namespace, type, model, payload, formName, field, formController, fieldDisplay, result;

    return _regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _KOSUtil$getActionTyp = KOSUtil$1.getActionType(action.type), namespace = _KOSUtil$getActionTyp.namespace, type = _KOSUtil$getActionTyp.type;
            model = KOS$1.getModel(namespace);
            payload = action.payload;
            formName = payload.formName, field = payload.field;
            formController = model.getAttr('formFieldDisplay') || {};

            if (formName) {
              formController = formController[formName] || {};
            }

            fieldDisplay = formController[field];

            if (!fieldDisplay) {
              _context.next = 13;
              break;
            }

            _context.next = 10;
            return fieldDisplay(getState, payload);

          case 10:
            result = _context.sent;


            // 触发表单校验结果更新
            result && dispatch({
              type: 'setState',
              payload: createDisplayPayload(payload, getState, result)
            });

            return _context.abrupt('return', result);

          case 13:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, _this$2);
  }));

  return function fieldDisplayMiddleware(_x, _x2, _x3) {
    return _ref.apply(this, arguments);
  };
}();

var FieldDiaplay = {
  /**
   * 表单字段展示中间件
   */
  fieldDisplayMiddleware: fieldDisplayMiddleware
};

var _this$3 = undefined;

var fieldDisplayMiddleware$1 = FieldDiaplay.fieldDisplayMiddleware;
var fieldValidateMiddleware$1 = FormValidator.fieldValidateMiddleware,
    formValidateMiddleware$1 = FormValidator.formValidateMiddleware;

var KOSUtil$2 = KOS$1.Util;

var fieldChangeHandlers = [];
var FormMiddleware = function FormMiddleware(store) {
  return function (next) {
    return function () {
      var _ref = asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee(action) {
        var _KOSUtil$getActionTyp, namespace, type, getState, dispatch;

        return _regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                _KOSUtil$getActionTyp = KOSUtil$2.getActionType(action.type), namespace = _KOSUtil$getActionTyp.namespace, type = _KOSUtil$getActionTyp.type;

                getState = function getState() {
                  return store.getState()[namespace];
                };

                dispatch = function dispatch(action) {
                  var type = action.type;

                  store.dispatch(_extends({}, action, {
                    type: namespace + '/' + type
                  }));
                };

                _context.next = 5;
                return next(action);

              case 5:
                _context.t0 = type;
                _context.next = _context.t0 === XFORM_FIELD_CHANGE ? 8 : _context.t0 === XFORM_VALIDATE ? 11 : 14;
                break;

              case 8:
                // 更新值
                dispatch({
                  type: 'setState',
                  payload: createFieldValuePayload(action.payload, getState)
                });

                fieldChangeHandlers.forEach(function (handler) {
                  handler(dispatch, getState, action);
                });
                return _context.abrupt('break', 14);

              case 11:
                _context.next = 13;
                return formValidateMiddleware$1(dispatch, getState, action);

              case 13:
                return _context.abrupt('break', 14);

              case 14:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, _this$3);
      }));

      return function (_x) {
        return _ref.apply(this, arguments);
      };
    }();
  };
};

FormMiddleware.addFieldChangeHandler = function (handler) {
  handler && fieldChangeHandlers.push(handler);
};

// 添加
FormMiddleware.addFieldChangeHandler(fieldDisplayMiddleware$1);
FormMiddleware.addFieldChangeHandler(fieldValidateMiddleware$1);

var Form$1 = Form;
var formMiddleware = FormMiddleware;
var FieldHOC = xField;
var addRule$2 = addRule;
var removeRule$1 = removeRule;

export { Form$1 as Form, formMiddleware, FieldHOC, addRule$2 as addRule, removeRule$1 as removeRule };
